#include <xc.h>
#include <stdbool.h>
#include "inputs.h"
#include "midi.h"
#include "adc.h"
#include "io_mapping.h"
#include "encoder.h"

#define _XTAL_FREQ 20000000

void setAdress(char adr)
{
    ADC_ADR_PORT = adr;
}


void procesAnalogInputs(char chanel)
{
    char adr, value, controller;

    for(adr = 0; adr < A_INPUTS; adr++){

        // Skip of empty 5th multiplexer input
        if(adr == 4) continue;
        
        value = getADC(chanel, adr);

        if(a_values[adr] > (value + 1) 
              || a_values[adr] < (value - 1)){
//        if(a_values[adr] != value){

            controller = controllers[adr];
        
            if(sendMIDIControl(controller, value)){
                a_values[adr] = value;
            }
        }
    }
}


void getButtons()
{
    char i;
    
    //Strobe PL to load data on all 4021 inputs
    BTN_PL = true;
    __delay_us(5);
    BTN_PL = false;
    
    BTN_CLOCK = false;
    
    for(i = 0; i < BUTTONS; i++){
        
        buttons[i] = BTN_DATA;
        
        // raise clock edge
        BTN_CLOCK = true;
        __delay_us(5);
        BTN_CLOCK = false;
    }
}

void sendButtons()
{
    char i, value, controller;
    
    for(i = 0; i < BUTTONS; i++){
        
        value = buttons[i];
        
        if(value != b_values[i]){

            // Buttons controller number start at 10
            controller = controllers[i + 10];
            
            if(sendMIDIControl(controller, value)){
                b_values[i] = value;
            }
        }
    }

}


void procesEncoder()
{
    char enc_direction, controller;
    
    // Process Encoder
    enc_direction = getEncDirection();
    
    if(enc_direction != 0){
        controller = controllers[ENC_CTRL];
        sendMIDIControl(controller, enc_direction);
    }
    
    if(enc_sw != ENC_SW){
        controller = controllers[ENC_SW_CTRL];
        sendMIDIControl(controller, enc_sw);
        enc_sw = ENC_SW;
    }
}


void procesJogWheel()
{
    char jog_direction, controller;
    
    // Process Encoder
    jog_direction = getJogDirection();
    
    if(jog_direction != 0){
        controller = controllers[JOG_CTRL];
        sendMIDIControl(controller, jog_direction);
    }
}

